local MODE_EPGP = "epgp"
local MODE_ROLL = "roll"
local currentMode = MODE_ROLL

local bufferedData = {}

local function LoadPlayerInfo(itemIdx, playerIdx)

    local rollID = C_LootHistory.GetItem(itemIdx);
    local name, _, rollType = C_LootHistory.GetPlayerInfo(itemIdx, playerIdx);
    local currentData

    if (not bufferedData[rollID]) then
        bufferedData[rollID] = {}
    end

    if (not bufferedData[rollID][name]) then
        local ep,gp = EPGP:GetEPGP(name)
        currentData = {}
        currentData.EPGP = 0
        currentData.hasNotMin = true
        if (ep and gp and gp > 0) then
            currentData.EPGP = (ep/gp)
        end
        if (ep) then
            currentData.hasNotMin = (EPGP:GetMinEP() > ep)
        end
        bufferedData[rollID][name] = currentData
    else
        currentData = bufferedData[rollID][name]
    end

    return currentData.EPGP, currentData.hasNotMin, rollType
end

local origUpdatePlayerFrame = LootHistoryFrame_UpdatePlayerFrame
function LootHistoryFrame_UpdatePlayerFrame(self, playerFrame)

    local origResult = origUpdatePlayerFrame(self,playerFrame)
    if (playerFrame.playerIdx) then
        if (currentMode == MODE_EPGP) then
            -- load player data
            local epgp, hasNotMinEp, rollType = LoadPlayerInfo(playerFrame.itemIdx, playerFrame.playerIdx)
            if (rollType and (rollType == LOOT_ROLL_TYPE_NEED or rollType == LOOT_ROLL_TYPE_GREED or rollType == LOOT_ROLL_TYPE_DISENCHANT)) then
                local text = string.format(" %.2f", epgp)
                if (hasNotMinEp == true) then
                    text = '|cff555555'..text..'|r'
                end

                playerFrame.RollText:SetText(text)
                playerFrame.RollText:SetSize(0,16)
            end
        else
            -- reset to default options
            playerFrame.RollText:SetSize(24,16)
        end
    end

    return origResult
end

local function comparePlayerFrames(f1, f2)
    if (f1.playerIdx and f2.playerIdx ) then

        local firstEPGP, firstHasNotMin, firstRollType = LoadPlayerInfo(f1.itemIdx, f1.playerIdx)
        local secondEPGP, secondHasNotMin, secondRollType = LoadPlayerInfo(f2.itemIdx, f2.playerIdx)

        if (firstRollType == secondRollType) then
            if ((firstHasNotMin == false and secondHasNotMin == true) or (firstEPGP > secondEPGP)) then
                return true
            end
        elseif (firstRollType == LOOT_ROLL_TYPE_NEED) then
            return true
        elseif (firstRollType == LOOT_ROLL_TYPE_GREED and (secondRollType == LOOT_ROLL_TYPE_DISENCHANT or secondRollType == LOOT_ROLL_TYPE_PASS)) then
            return true
        elseif (firstRollType == LOOT_ROLL_TYPE_DISENCHANT and secondRollType == LOOT_ROLL_TYPE_PASS) then
            return true
        end

    elseif (f1.playerIdx) then
        return true
    elseif (not f1.playerIdx and not f2.playerIdx) then
        return true
    end
end

local origUpdatePlayerFrames = LootHistoryFrame_UpdatePlayerFrames
function LootHistoryFrame_UpdatePlayerFrames(self, itemIdx)

    local firstFrame, lastFrame = origUpdatePlayerFrames(self, itemIdx)

    if (currentMode == MODE_EPGP) then
       local frameList = {}

       -- build lookup table of player frames
       local currentFrame = lastFrame
       repeat
           if (currentFrame.playerIdx) then
               table.insert(frameList, currentFrame)
           end
           local pointCount = currentFrame:GetNumPoints()
           if (pointCount > 0) then
               local point, relativeTo = currentFrame:GetPoint(n)
               currentFrame = relativeTo
           end
       until (pointCount == 0)

       table.sort(frameList, comparePlayerFrames)

       firstFrame = nil
       lastFrame = nil
       for i=1,#frameList do
           local playerFrame = frameList[i]
           playerFrame:ClearAllPoints()
           if ( lastFrame ) then
               playerFrame:SetPoint("TOPLEFT", lastFrame, "BOTTOMLEFT", 0, -1);
           end
           if ( i % 2 == 1) then
               playerFrame.AlternatingBG:Show();
           else
               playerFrame.AlternatingBG:Hide();
           end

           lastFrame = playerFrame
           firstFrame = firstFrame or playerFrame;
       end
    end

    return firstFrame, lastFrame
end

local function UpdateBtnTexture(btn, mode)

    if (mode == MODE_EPGP) then
        btn:SetNormalTexture("Interface\\GossipFrame\\VendorGossipIcon")
    else
        btn:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Up")
    end
end

local function ToggleCurrentMode(self)

    if (currentMode == MODE_ROLL) then
        currentMode = MODE_EPGP
    else
        currentMode = MODE_ROLL
    end

    UpdateBtnTexture(self, currentMode)
    LootHistoryFrame_FullUpdate(LootHistoryFrame)
end

local function BuildToggleBtn(frame)

    local btn = CreateFrame("Button","ToggleLootModeBtn",frame)
    btn:SetPoint("TOPRIGHT",frame,"TOPLEFT",2,-8)
    btn:SetSize(16, 16)
    UpdateBtnTexture(btn, currentMode)
    btn:SetScript('OnClick', ToggleCurrentMode)

    return btn
end

LootHistoryFrame:HookScript("OnShow", function(self)
    if (not self.ToggleLootModeBtn) then
        self.ToggleLootModeBtn = BuildToggleBtn(self.CloseButton)
    end
end)
